package owl-base

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Neuron.EmbeddingSource

Sourcetype neuron_typ = {
  1. mutable w : Optimise.Algodiff.t;
  2. mutable init_typ : Init.typ;
  3. mutable in_dim : int;
  4. mutable in_shape : int array;
  5. mutable out_shape : int array;
}

Neuron type definition.

Sourceval create : ?inputs:int -> int -> int -> Init.typ -> neuron_typ

Create the neuron.

Sourceval connect : int array -> neuron_typ -> unit

Connect this neuron to others in a neural network.

Sourceval init : neuron_typ -> unit

Initialise the neuron and its parameters.

Sourceval reset : neuron_typ -> unit

Reset the parameters in a neuron.

Sourceval mktag : int -> neuron_typ -> unit

Tag the neuron, used by ``Algodiff`` module.

Assemble all the parameters in an array, used by ``Optimise`` module.

Assemble all the primial values in an array, used by ``Optimise`` module.

Assemble all the adjacent values in an array, used by ``Optimise`` module.

Sourceval update : neuron_typ -> Optimise.Algodiff.t array -> unit

Update parameters in a neuron, used by ``Optimise`` module.

Make a deep copy of the neuron and its parameters.

Execute the computation in this neuron.

Sourceval to_string : neuron_typ -> string

Convert the neuron to its string representation. The string is often a summary of the parameters defined in the neuron.

Sourceval to_name : unit -> string

Return the name of the neuron.