package owl-base

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Neuron.NormalisationSource

Sourcetype neuron_typ = {
  1. mutable axis : int;
  2. mutable beta : Optimise.Algodiff.t;
  3. mutable gamma : Optimise.Algodiff.t;
  4. mutable mu : Optimise.Algodiff.t;
  5. mutable var : Optimise.Algodiff.t;
  6. mutable decay : Optimise.Algodiff.t;
  7. mutable training : bool;
  8. mutable in_shape : int array;
  9. mutable out_shape : int array;
}

Neuron type definition.

Sourceval create : ?training:bool -> ?decay:float -> ?mu:Optimise.Algodiff.A.arr -> ?var:Optimise.Algodiff.A.arr -> int -> neuron_typ

Create the neuron. Note that axis 0 is the batch axis.

Sourceval connect : int array -> neuron_typ -> unit

Connect this neuron to others in a neural network.

Sourceval init : neuron_typ -> unit

Initialise the neuron and its parameters.

Sourceval reset : neuron_typ -> unit

Reset the parameters in a neuron.

Sourceval mktag : int -> neuron_typ -> unit

Tag the neuron, used by Algodiff module.

Assemble all the trainable parameters in an array, used by Optimise module.

Assemble all the primial values in an array, used by Optimise module.

Assemble all the adjacent values in an array, used by Optimise module.

Sourceval update : neuron_typ -> Optimise.Algodiff.t array -> unit

Update trainable parameters of the neuron, used by Optimise module.

Sourceval load_weights : neuron_typ -> Optimise.Algodiff.t array -> unit

Load both trainable and non-trainable parameters into the neuron.

Sourceval save_weights : neuron_typ -> Optimise.Algodiff.t array

Assemble both trainable and non-trainable parameters of the neuron.

Make a deep copy of the neuron and its parameters.

Execute the computation in this neuron.

Sourceval to_string : neuron_typ -> string

Convert the neuron to its string representation. The string is often a summary of the parameters defined in the neuron.

Sourceval to_name : unit -> string

Return the name of the neuron.