package owl-base

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Owl_types_operatorSource

Operator definitions such as add, sub, mul, and div. This signature defines the functions need to be implemented.

Sourcemodule type BasicSig = sig ... end
Sourcemodule type ExtendSig = sig ... end
Sourcemodule type MatrixSig = sig ... end
Sourcemodule type NdarraySig = sig ... end
Sourcemodule type LinalgSig = sig ... end